var CHAR_UPPER_A = 0x41;
var CHAR_LOWER_A = 0x61;
var CHAR_UPPER_Z = 0x5a;
var CHAR_LOWER_Z = 0x7a;
var CHAR_0 = 0x30;
var CHAR_9 = 0x39;
var CHAR_MINUS = 0x2d;
var CHAR_SPACE = 0x20;
var CHAR_UNDERSCORE = 0x5f;

function isUpper(c) {
  return CHAR_UPPER_A <= c && c <= CHAR_UPPER_Z;
}

function isLower(c) {
  return CHAR_LOWER_A <= c && c <= CHAR_LOWER_Z;
}

function isDigit(c) {
  return CHAR_0 <= c && c <= CHAR_9;
}

function toUpper(c) {
  return c - 0x20;
}

function toLower(c) {
  return c + 0x20;
}

function camelize(str) {
  var firstChar = str.charCodeAt(0);

  if (isDigit(firstChar) || firstChar === CHAR_MINUS || isNaN(firstChar)) {
    return str;
  }

  var changed = isUpper(firstChar);
  var transformed = changed ? [toLower(firstChar)] : [firstChar];
  var length = str.length;

  for (var i = 1; i < length; i++) {
    var c = str.charCodeAt(i);

    if (c === CHAR_UNDERSCORE || c === CHAR_SPACE || c === CHAR_MINUS) {
      changed = true;
      c = str.charCodeAt(++i);

      if (isNaN(c)) {
        return str;
      }

      if (isLower(c)) {
        transformed.push(toUpper(c));
      } else {
        transformed.push(c);
      }
    } else {
      transformed.push(c);
    }
  }

  if (!changed) {
    return str;
  }

  return String.fromCharCode.apply(undefined, transformed);
}
function decamelize(str) {
  var firstChar = str.charCodeAt(0);

  if (!isLower(firstChar) || isNaN(firstChar)) {
    return str;
  }

  var changed = false;
  var transformed = [firstChar];
  var length = str.length;

  for (var i = 1; i < length; i++) {
    var c = str.charCodeAt(i);

    if (isUpper(c)) {
      changed = true;
      transformed.push(CHAR_UNDERSCORE);
      transformed.push(toLower(c));
    } else {
      transformed.push(c);
    }
  }

  if (!changed) {
    return str;
  }

  return String.fromCharCode.apply(undefined, transformed);
}
function pascalize(str) {
  var firstChar = str.charCodeAt(0);

  if (isDigit(firstChar) || firstChar === CHAR_MINUS || isNaN(firstChar)) {
    return str;
  }

  var changed = isLower(firstChar);
  var transformed = changed ? [toUpper(firstChar)] : [firstChar];
  var len = str.length;

  for (var i = 1; i < len; i++) {
    var c = str.charCodeAt(i);

    if (c === CHAR_UNDERSCORE || c === CHAR_SPACE || c === CHAR_MINUS) {
      changed = true;
      c = str.charCodeAt(++i);

      if (isNaN(c)) {
        return str;
      }

      if (isLower(c)) {
        transformed.push(toUpper(c));
      } else {
        transformed.push(c);
      }
    } else {
      transformed.push(c);
    }
  }

  if (!changed) {
    return str;
  }

  return String.fromCharCode.apply(undefined, transformed);
}
function depascalize(str) {
  var firstChar = str.charCodeAt(0);

  if (!isUpper(firstChar) || isNaN(firstChar)) {
    return str;
  }

  var transformed = [toLower(firstChar)];
  var length = str.length;

  for (var i = 1; i < length; i++) {
    var c = str.charCodeAt(i);

    if (isUpper(c)) {
      transformed.push(CHAR_UNDERSCORE);
      transformed.push(toLower(c));
    } else {
      transformed.push(c);
    }
  }

  return String.fromCharCode.apply(undefined, transformed);
}

function isPlainObject(value) {
  return value && typeof value === 'object' && !(value instanceof Function) && !(value instanceof Date);
}

function transformKeys(obj, transform, inPlace) {
  if ( inPlace === void 0 ) inPlace = false;

  var transformed;

  if (inPlace) {
    transformed = obj;
  } else {
    if (Array.isArray(obj)) {
      transformed = [];
    } else {
      if (typeof obj.prototype !== 'undefined') {
        return obj;
      }

      transformed = {};
    }
  }

  for (var key in obj) {
    var value = obj[key];
    var nextKey = key;

    if (typeof key === 'string') {
      nextKey = transform(key);
    }

    if (inPlace && nextKey !== key) {
      delete transformed[key];
    }

    if (isPlainObject(value)) {
      transformed[nextKey] = transformKeys(value, transform, inPlace);
    } else {
      transformed[nextKey] = value;
    }
  }

  return transformed;
}

function camelizeKeys(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, camelize);
}
function decamelizeKeys(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, decamelize);
}
function pascalizeKeys(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, pascalize);
}
function depascalizeKeys(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, depascalize);
}
function camelizeKeysInPlace(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, camelize, true);
}
function decamelizeKeysInPlace(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, decamelize, true);
}
function pascalizeKeysInPlace(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, pascalize, true);
}
function depascalizeKeysInPlace(obj) {
  if (!isPlainObject(obj)) {
    return obj;
  }

  return transformKeys(obj, depascalize, true);
}

export { camelize, decamelize, pascalize, depascalize, camelizeKeys, decamelizeKeys, pascalizeKeys, depascalizeKeys, camelizeKeysInPlace, decamelizeKeysInPlace, pascalizeKeysInPlace, depascalizeKeysInPlace };
